import { Injectable } from '@angular/core';
import { KmsApi } from 'src/app/apis/kms.service';
import { BlockCardStore } from './block-card-store.service';
import { StorageService } from 'src/app/storage/storage-service.service';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import { AlertController } from '@ionic/angular';

@Injectable({
  providedIn: 'root'
})
export class BlockCardFacade {

  // Texts for alert prompts:
  confirmHeader: string;
  confirmText: string;
  statusText: string;
  passwordText: string;
  passwordErrorTitle: string;
  passwordErrorText: string;
  serverErrorTitle: string;
  serverErrorText: string;
  cardBlockedText: string;
  successText: string;

  constructor(private store: BlockCardStore, private storage: StorageService, private kmsApi: KmsApi, private router: Router, private translateService: TranslateService, private alertController: AlertController) {}

  getUni() { return this.storage.select(state => state.SELECTED_UNI); }
  isCardBlocked() { return this.storage.select(state => state.CARD_IS_BLOCKED); }
  isPasswordConfirmInProgress() { return this.store.isPasswordConfirmInProgress(); }

  initialize() {
    this.store.setPasswordConfirmInProgress(false);
    this.storage.setIsCardBlocked(false);
  }

  getTranslations() {
    this.translateService.get('blockCard').subscribe(blockCard => {
      this.confirmHeader = blockCard.title;
      this.confirmText = blockCard.confirm;
      this.passwordText = blockCard.password;
      this.passwordErrorTitle = blockCard.passwordError.header;
      this.passwordErrorText = blockCard.passwordError.text;
      this.serverErrorTitle = blockCard.serverError.header;
      this.serverErrorText = blockCard.serverError.text;
      this.cardBlockedText = blockCard.cardBlocked;
      this.successText = blockCard.done;
    })
  }

  async presentAlertPrompt(header, message) {
    const alert = await this.alertController.create({
      header: header,
      message: message,
      inputs: [
        {
          name: 'passwordInput',
          type: 'password',
          placeholder: this.passwordText,
          attributes: {
            maxlength: 30,
            inputmode: 'text'
          }
        }
      ],
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel'
        }, {
          text: 'OK',
          handler: (input) => {
            this.passwordConfirm(input);
          }
        }
      ]
    });
    await alert.present();
  }

  async presentSuccessAlert() {
    const alert = await this.alertController.create({
      header: this.cardBlockedText,
      message: this.successText,
      buttons: [
        {
          text: 'OK',
          handler: () => {
            this.router.navigate(['..']) // back to myCard page
          }
        }
      ]
    });
    await alert.present();
  }

  async passwordConfirm(password) {
    
    // Password empty:
    if (password.passwordInput == '') {
      this.presentAlertPrompt(this.confirmHeader, this.confirmText);
      this.store.setPasswordConfirmInProgress(false);
    } else {

      // Passwort not empty:
      this.store.setPasswordConfirmInProgress(true);
      await this.kmsApi.passwordConfirm(password.passwordInput)
      .then(() => {
        this.storage.setIsCardBlocked(true);
        this.presentSuccessAlert();
      })
      .catch(error => {
        // Wrong password:
        if (error.status == 401) {
          this.presentAlertPrompt(this.passwordErrorTitle, this.passwordErrorText);
        // Server error:
        } else if (error.status > 499) {
          this.presentAlertPrompt(this.serverErrorTitle, this.serverErrorText);
        // Anything else => open same prompt again:
        } else {
          this.presentAlertPrompt(this.confirmHeader, this.confirmText);
        }
      })
      .finally(() => this.store.setPasswordConfirmInProgress(false));
    }
  }

}
