import { Component, OnDestroy } from '@angular/core';
import { IonRefresher, ViewWillEnter } from '@ionic/angular';
import { TranslateService } from '@ngx-translate/core';
import { Observable, Subscription } from 'rxjs';
import { CardData, Uni } from 'src/app/types';
import { MyCardFacade } from './my-card-facade.service';
import { localeID } from 'src/assets/i18n/languages';
import { StorageService } from 'src/app/storage/storage-service.service';
import { MenuPage } from '../menu.page';

// Cache expiry time:
const ONE_DAY = 24 * 60 * 60 * 1000;

// Interval for checking if the content in view is older than one day:
const CHECK_INTERVAL = 60 * 1000;

@Component({
  selector: 'app-my-card',
  templateUrl: './my-card.page.html',
  styleUrls: ['./my-card.page.scss'],
})
export class MyCardPage implements ViewWillEnter, OnDestroy {

  selectedUni: Observable<Uni>;

  cardData: Observable<CardData>;
  photo: Observable<String>;
  isCardDataUpdateFailed: Observable<boolean>;
  isCardDataUpdateInProgress: Observable<boolean>;
  isCardBlocked: Observable<boolean>;
  escIdentifier: Observable<string>;

  lastCardDataFetchDate: Date; // Date of the last successful card data update
  lastCardDataFetchTime: number;
  dateSubscription: Subscription;

  locID: string;
  localFetchDate: string;
  expiryInterval: any;

  constructor(private facade: MyCardFacade,
    public menuPage: MenuPage,
    private localeID: localeID,
    private translateService: TranslateService,
    private storage: StorageService,
    ) {

    this.selectedUni = this.facade.getSelectedUni();

    this.cardData = this.facade.getCardData();
    this.photo = this.facade.getPhoto();
    this.isCardDataUpdateFailed = this.facade.isCardDataUpdateFailed();
    this.isCardDataUpdateInProgress = this.facade.isCardDataUpdateInProgress();
    this.isCardBlocked = this.facade.isCardBlocked();
    this.escIdentifier = this.facade.getEscIdentifier();

    this.dateSubscription = this.facade.getLastCardDataFetchDate().subscribe(date => {
      this.lastCardDataFetchDate = date;
      this.lastCardDataFetchTime = date.getTime();
    });

  }

  // Check if last card data fetch was more than one day ago:
  private cardDataExpired() {
    return new Date().getTime() > this.lastCardDataFetchTime + ONE_DAY
  }

  // Check this regularly:
  private expiryCheck() {
    this.expiryInterval = setInterval(this.cardDataExpired, CHECK_INTERVAL);
  }

  // Drag-Down-Refresher will refresh and start new expiry check:
  doRefresh(event: CustomEvent) {
    const refresher = event.target as unknown as IonRefresher;
    this.facade.doRefresh(refresher);
    this.update();
    refresher?.complete()
  }

  // Localize date for current language:
  private localizeDate() {
    this.locID = this.localeID.getLocalID(this.translateService.currentLang);
    this.localFetchDate = this.lastCardDataFetchDate.toLocaleDateString(this.locID, { day: '2-digit', month: '2-digit', year: '2-digit', hour: '2-digit', minute: '2-digit', second: '2-digit' })
  }

  // Update via click, via drag-down-refresher, or via navigation if cache expired:
  update() {
    clearInterval(this.expiryInterval);
    this.expiryCheck();
    this.facade.doRefresh();
    // Localize date and display:
    this.localizeDate();
  }

  // On entering the view:
  ionViewWillEnter() {
    // Get ESC data:
    this.facade.loadEscData();
    // If storage is empty, card data expired, or the fetch date is not displayed, refresh and start new expiry check:
    Promise.all([this.storage.getCardData()]).then(dataInStorage => {
      if (dataInStorage.every(entry => !entry) || this.cardDataExpired() || !this.localFetchDate) {
        this.update();
      }
    });
  }

  ngOnDestroy() {
    clearInterval(this.expiryInterval);
  }

}
